# encoding: UTF-8

require 'rubygems'
gem 'shoulda'
require 'test/unit'
require 'shoulda'
require 'blabla_client/trip'

class TripTest < Test::Unit::TestCase

  RESPONSE_BODY =
      '{"links":{"_self":"http://api.blablacar.fr/api/v2/trips"},"pager":{"page":1,"pages":68,"total":674,"limit":10},' +
      '"trips":[' +
      '{"links":{"_self":"http://api.blablacar.fr/api/v2/trips/95946679-lyon-grenoble","_front":"http://www.covoiturage.fr/trajet-lyon-grenoble-95946679","_threads":"http://api.blablacar.fr/api/v2/trips/95946679-lyon-grenoble/threads"},"frequency":"UNIQUE","departure_date":"15/02/2015 15:15:00","departure_place":{"city_name":"Lyon","address":"Lyon","latitude":45.764043,"longitude":4.835659},"arrival_place":{"city_name":"Grenoble","address":"Grenoble","latitude":45.188529,"longitude":5.724524},"price":{"value":7,"currency":"EUR","symbol":"\u20ac","string_value":"7\u00a0\u20ac","price_color":"GREEN"},"seats_left":0,"seats":2,"duration":{"value":3600,"unity":"s"},"distance":{"value":111,"unity":"km"},"permanent_id":"95946679-lyon-grenoble","car":{"model":"SCENIC III","make":"RENAULT","comfort":"Confort","comfort_nb_star":3},"viaggio_rosa":false,"freeway":true,"answer_delay":3,"booking_mode":"manual","booking_type":"online"},' +
      '{"links":{"_self":"http://api.blablacar.fr/api/v2/trips/98529037-lyon-grenoble","_front":"http://www.covoiturage.fr/trajet-lyon-grenoble-98529037","_threads":"http://api.blablacar.fr/api/v2/trips/98529037-lyon-grenoble/threads"},"frequency":"UNIQUE","departure_date":"15/02/2015 15:20:00","departure_place":{"city_name":"Lyon","address":"Lyon","latitude":45.7640299,"longitude":4.8350211},"arrival_place":{"city_name":"Grenoble","address":"Grenoble","latitude":45.188529,"longitude":5.724524},"price":{"value":7,"currency":"EUR","symbol":"\u20ac","string_value":"7\u00a0\u20ac","price_color":"GREEN"},"seats_left":1,"seats":3,"duration":{"value":4800,"unity":"s"},"distance":{"value":111,"unity":"km"},"permanent_id":"98529037-lyon-grenoble","car":{"model":"PASSAT","make":"VOLKSWAGEN","comfort":"Confort","comfort_nb_star":3},"viaggio_rosa":false,"freeway":true,"answer_delay":1,"booking_mode":"manual","booking_type":"online"},' +
      '{"links":{"_self":"http://api.blablacar.fr/api/v2/trips/97965469-lyon-grenoble","_front":"http://www.covoiturage.fr/trajet-lyon-grenoble-97965469","_threads":"http://api.blablacar.fr/api/v2/trips/97965469-lyon-grenoble/threads"},"frequency":"UNIQUE","departure_date":"15/02/2015 15:30:00","departure_place":{"city_name":"Lyon","address":"Place Jean Jaur\u00e8s, Lyon","latitude":45.7390393,"longitude":4.8395687},"arrival_place":{"city_name":"Grenoble","address":"Grenoble","latitude":45.188529,"longitude":5.724524},"price":{"value":5,"currency":"EUR","symbol":"\u20ac","string_value":"5\u00a0\u20ac","price_color":"GREEN"},"seats_left":2,"seats":4,"duration":{"value":6000,"unity":"s"},"distance":{"value":105,"unity":"km"},"permanent_id":"97965469-lyon-grenoble","car":{"model":"206","make":"PEUGEOT","comfort":"Normal","comfort_nb_star":2},"viaggio_rosa":false,"freeway":true,"answer_delay":6,"booking_mode":"manual","booking_type":"online"}' +
      ']}'

  should 'populate trips objects with the parsed json response' do
    trips = Trip.parse_response(RESPONSE_BODY)[:trips]

    assert_equal 3, trips.length
    assert_equal ['15/02/2015 15:15:00', '15/02/2015 15:20:00', '15/02/2015 15:30:00'], trips.collect {|t| t.departure_date}
    assert_equal [{:value => 7, :currency => '€', :color => 'green'}, {:value => 7, :currency => '€', :color => 'green'},
                  {:value => 5, :currency => '€', :color => 'green'}], trips.collect {|t| t.get_price}
    assert_equal ['Lyon', 'Lyon', 'Place Jean Jaurès, Lyon'], trips.collect {|t| t.get_departure_place}
    assert_equal ['Grenoble', 'Grenoble', 'Grenoble'], trips.collect {|t| t.get_arrival_place}
    assert_equal [0, 1, 2], trips.collect {|t| t.seats_left}
    assert_equal [{:hours => 1, :minutes => 0}, {:hours => 1, :minutes => 20}, {:hours => 1, :minutes => 40}], trips.collect {|t| t.get_duration}
    assert_equal [111, 111, 105], trips.collect {|t| t.get_distance}
  end

  should 'return departure date as a time object' do
    assert_equal Time.new(2015, 2, 15, 15, 15, 0), Trip.new({:departure_date => '15/02/2015 15:15:00'}).get_departure_date
  end

  should 'return the trip url' do
    assert_equal 'http://www.covoiturage.fr/trajet-lyon-grenoble-95946679', Trip.new({:links => {:_front => 'http://www.covoiturage.fr/trajet-lyon-grenoble-95946679'}}).get_url
  end

  should 'return paging fields included in the response' do
    response = Trip.parse_response(RESPONSE_BODY)

    assert_equal 1, response[:page]
    assert_equal 68, response[:pages]
    assert_equal 674, response[:count]
  end

  should 'return a trip duration hash' do
    assert_equal({:hours => 1, :minutes => 0}, Trip.new({:duration => {:value => 3600, :unity => 's'}}).get_duration)
    assert_equal({:hours => 2, :minutes => 0}, Trip.new({:duration => {:value => 7200, :unity => 's'}}).get_duration)
    assert_equal({:hours => 1, :minutes => 30}, Trip.new({:duration => {:value => 5400, :unity => 's'}}).get_duration)
    assert_equal({:hours => 0, :minutes => 40}, Trip.new({:duration => {:value => 2400, :unity => 's'}}).get_duration)
    assert_equal({:hours => 0, :minutes => 45}, Trip.new({:duration => {:value => 2710, :unity => 's'}}).get_duration)
    assert_equal({:hours => 1, :minutes => 0}, Trip.new({:duration => {:value => 3620, :unity => 's'}}).get_duration)
  end

  should 'return a data hash with the car information' do
    assert_equal({:model => 'RENAULT SCENIC III', :comfort => 3},
                 Trip.new({:car => {:model => 'SCENIC III', :make => 'RENAULT', :comfort => 'Confort', :comfort_nb_star => 3}}).get_car)
  end
end

